<?php
namespace Modules\Cadastros\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Modules\Cadastros\Services\ConfigAlcadasService;
use Modules\Cadastros\Services\FilialAlcadasService;
use Modules\Cadastros\Services\AprovadorService;
use Modules\Cadastros\Services\ConfigAlcadasSheetImporter;
use Zeedhi\Framework\DataSource\DataSet;

class ConfigAlcadasController
{
    const CHAVES_PARA_IGNORAR = array(
        '$error',
        '__createdLocal',
        '__is_new',
        '__index',
        '__isSelected',
        'changed',
    );

    public function __construct(
        ConfigAlcadasService $service,
        FilialAlcadasService $serviceFilialAlcada,
        AprovadorService $serviceAprovador,
        ConfigAlcadasSheetImporter $configAlcadasSheetImporter,
        $smtpUtil
    ) {
        $this->service             = $service;
        $this->serviceFilialAlcada = $serviceFilialAlcada;
        $this->serviceAprovador    = $serviceAprovador;
        $this->sheetImporter       = $configAlcadasSheetImporter;
        $this->smtpUtil            = $smtpUtil;
    }

    public function saveNewConfigAlcada(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $row = self::removerChavesIgnoradas($row);
        $row = self::verificarTipoAprovacao($row);

        try {
            $this->service->saveConfigAlcada($row);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function saveEditConfigAlcada(DTO\Request\Row $request, DTO\Response $response)
    {
        $row                      = $request->getRow();
        $updatedConfigAlcada      = $row['NEW'];
        $oldConfigAlcada          = $row['OLD'];

        try {
            $propriedadesAlteradas = self::getPropriedadesAlteradas($updatedConfigAlcada, $oldConfigAlcada);

            $qtdPropriedadesAlteradas = count($propriedadesAlteradas);

            if ($qtdPropriedadesAlteradas == 0) {
                throw new \Exception('THERE_ARE_NO_CHANGES');
            }

            $numeroConfigAlcada = $updatedConfigAlcada['NRCONFIGALCADA'];

            if (self::possuiAprovacoesPendentes($numeroConfigAlcada)) {
                if (!array_key_exists('NMALCADA', $propriedadesAlteradas) || $qtdPropriedadesAlteradas > 1) {
                    throw new \Exception('OPERATION_BLOCKED_SAVE_COMPETENCE_PENDING_APPROVALS');
                }
            }

            $isTipoAlcadaAlterado     = array_key_exists('IDTPALCADA', $propriedadesAlteradas);
            $isExigeAprovacaoAlterado = array_key_exists('IDEXIGETAPROVE', $propriedadesAlteradas);
            $isTipoAprovacao          = array_key_exists('IDTIPOVRAPROV', $propriedadesAlteradas);

            if ($isTipoAlcadaAlterado || $isExigeAprovacaoAlterado || $isTipoAprovacao ) {
                $this->serviceAprovador->deleteTodosAprovadoresRelacionadosAlcada($updatedConfigAlcada);
                $this->serviceAprovador->deleteTodosAprovadoresAlcada($updatedConfigAlcada);
            }
            $this->service->updateConfigAlcada($updatedConfigAlcada);

            /**
             * A função abaixo é a forma padrão de salvar o log, porém, atualmente, os logs são salvos em arquivos
             * em vez de banco. Porém, essa funcionalidade depende da criação da tabela de log e, portanto,
             * encontra-se comentada até que seja filizada sua implementação no ConfigAlcadasService.
             *
             * A função que a substitui se chama saveLogAlteracaoArquivo, encontrada no service.
             */
            // $this->salvarAlteracoesEmLog($numeroConfigAlcada, $propriedadesAlteradas);

            // $this->service->saveLogAlteracaoArquivo($numeroConfigAlcada, $propriedadesAlteradas);

        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function deleteConfigAlcada(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();

        try {
            if (self::possuiAprovacoesPendentes($row['NRCONFIGALCADA'])) {
                throw new \Exception('OPERATION_BLOCKED_DEL_COMPETENCE_PENDING_APPROVALS');
            }

            $this->serviceAprovador->deleteTodosAprovadoresRelacionadosAlcada($row);
            $this->serviceAprovador->deleteTodosAprovadoresAlcada($row);
            $this->serviceFilialAlcada->deleteTodasFiliaisAlcada($row);
            $this->service->deleteConfigAlcada($row);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }


    public function atualizarAprovador(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();

        try {
            $this->serviceAprovador->deleteTodosAprovadoresRelacionadosAoAprovador($row);
            $this->serviceAprovador->changeAprovador($row);
            $this->serviceAprovador->updateAprovadorPendente($row);
            $this->serviceAprovador->deleteRelacionamentoAprovadorFilialSupplyTek($row);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    /**
     * @salvarAlteracoesEmLog
     * Atualmente, os logs são salvos em arquivos em vez de banco. O padrão
     * é ser salvo em banco, porém, depende da criação da tabela de log.
     *
     * A função saveLogAlteracao do service já está criada de forma incompleta.
     * Sua funcionalidade será implementada assim que houver os recursos necessários.
     */
    // public function salvarAlteracoesEmLog($numeroConfigAlcada, $propriedadesAlteradas)
    // {
    //     foreach ($propriedadesAlteradas as $nomePropriedade => $fraseDeAlteracao) {
    //         $this->service->saveLogAlteracao($numeroConfigAlcada, $nomePropriedade, $fraseDeAlteracao);
    //     }
    // }

    public function possuiAprovacoesPendentes($NRCONFIGALCADA)
    {
        return $this->service->findAlcada($NRCONFIGALCADA)['APROVACAO'] > 0;
    }

    public function removerChavesIgnoradas($row)
    {
        if (method_exists($row, 'getArrayCopy')) {
            $row = array_filter($row->getArrayCopy());
        }

        $row = array_diff_key($row, array_fill_keys(self::CHAVES_PARA_IGNORAR, ''));

        return $row;
    }

    public function verificarTipoAprovacao($row)
    {
        if(!isset($row['IDTIPOVRAPROV'])) {
            $row['IDTIPOVRAPROV'] = "";
        }

        return $row;
    }


    public function getPropriedadesAlteradas($updatedConfigAlcada, $oldConfigAlcada)
    {
        $propriedadesAlteradas = array();

        foreach ($updatedConfigAlcada as $key => $newValue) {
            if (array_key_exists($key, $oldConfigAlcada)) {
                $oldValue = $oldConfigAlcada[$key];

                if ($newValue != $oldValue) {
                    $propriedadesAlteradas[$key] = "alterada de '" . $oldValue . "' para '" . $newValue . "'";
                }
            }
        }

        return $propriedadesAlteradas;
    }


    function getIdSisDos(DTO\Request\Row $request, DTO\Response $response)
    {
        $idSisDos = $this->service->getIdSisDos();

        $response->addDataSet(new DataSet("callback", $idSisDos));
    }

    public function importSpreadsheet(DTO\Request $request, DTO\Response $response)
    {
		try {
		    $this->sheetImporter->beginTransaction();
			$row = $request->getRow();

			$sheetPath = $this->sheetImporter->createTemporaryFile($row["SPREADSHEET"]);
			$worksheet = $this->sheetImporter->loadSheet($sheetPath);
			$sheet = $this->sheetImporter->parseSheet($worksheet);

			$this->sheetImporter->validateSheetColumns($sheet);
			$this->sheetImporter->validateSheetRows($sheet, $response);

			$sequential = $this->sheetImporter->validateAndInsertData($sheet, $response);

			$this->sheetImporter->startImportProcedure($sequential);
			$this->sheetImporter->commitTransaction();

		    $errors = $this->sheetImporter->checkForErrors($sequential);


		    if (isset($errors[0])) {
		        $response->addDataSet(new DataSet("errors", $errors));
		    }
		} catch (\Exception $e) {
		    $this->sheetImporter->rollbackTransaction();
			$response->setError(new Error($e->getMessage(), 500));
		} finally {
			if (isset($sheetPath)) {
                $this->sheetImporter->removeTemporaryFile($sheetPath);
            }
		}
    }

    public function verificaItemOrcamento(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $validation = $this->service->verificaItemOrcamento($row);
        $response->addDataSet(new DataSet("verificaItem", $validation));
    }
}
